﻿using MSHelpMeHD.Data;

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Graphics.Display;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Items Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234233

namespace MSHelpMeHD
{
    using MSHelpMeHD.Common;

    using Windows.System;
    using Windows.UI.Popups;

    /// <summary>
    /// A page that displays a collection of item previews.  In the Split Application this page
    /// is used to display and select one of the available groups.
    /// </summary>
    public sealed partial class ItemsPage : MSHelpMeHD.Common.LayoutAwarePage
    {
        public ItemsPage()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// Populates the page with content passed during navigation.  Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="navigationParameter">The parameter value passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested.
        /// </param>
        /// <param name="pageState">A dictionary of state preserved by this page during an earlier
        /// session.  This will be null the first time a page is visited.</param>
        protected override void LoadState(Object navigationParameter, Dictionary<String, Object> pageState)
        {
            this.DefaultViewModel["Items"] = SampleDataSource.Instance.FlaggedItems;
            this.DefaultViewModel["SampleSource"] = SampleDataSource.Instance;

            if (!string.IsNullOrWhiteSpace(SampleDataSource.Instance.QueryText))
            {
                this.txtSearchText.Text = SampleDataSource.Instance.QueryText;
            }
        }

        /// <summary>
        /// Invoked when an item is clicked.
        /// </summary>
        /// <param name="sender">The GridView (or ListView when the application is snapped)
        /// displaying the item clicked.</param>
        /// <param name="e">Event data that describes the item clicked.</param>
        private void ItemView_ItemClick(object sender, ItemClickEventArgs e)
        {
            // Navigate to the appropriate destination page, configuring the new page
            // by passing required information as a navigation parameter
            var groupId = ((SampleDataItem)e.ClickedItem).Uri;
            this.Frame.Navigate(typeof(FavoritesPage), groupId);
        }

        private void txtSearchText_KeyUp(object sender, Windows.UI.Xaml.Input.KeyRoutedEventArgs e)
        {
            if (e.Key == VirtualKey.Enter && !string.IsNullOrWhiteSpace(txtSearchText.Text))
            {
                this.btnSearch_Click(sender, null);
            }
        }

        private void txtSearchText_TextChanged(object sender, Windows.UI.Xaml.Controls.TextChangedEventArgs e)
        {
            btnSearch.IsEnabled = !string.IsNullOrWhiteSpace(txtSearchText.Text);
        }

        private void txtSearchText_GotFocus(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            txtSearchText.SelectAll();
        }

        private void btnSearch_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(SplitPage), this.txtSearchText.Text);
        }

        private void btnTakePicture_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
        }
    }
}
